set_plot,'PS'

if !d.name eq 'PS' then device,xsize=8.,ysize=6.,/inches,$
/portrait,xoffset=1.,yoffset=1.,/color,bits=24,filename='dens.ps'

;makes three plots at 3 different radii in the disk/envelope
;set the inner and outer radii, and directory
rmin=.05
rmax1=0.3
rmax2=20.
rmax3=5000.
rhomin1=2.e-13 ;minimum density to be plotted.
rhomin2=2.e-17
rhomin3=2.e-20
range=5        ;range of decades plotted (log(dens) is plotted)
dir='../runenv/'

nr=1L
nt=1L
np=1L


close,1
openr,1,dir+'darr.unf',/f77_unformatted
readu,1,nr,nt,np
r=dblarr(nr)
theta=dblarr(nt)
phi=dblarr(np)
rho=dblarr(nr,nt,np)
readu,1,r
readu,1,theta
readu,1,phi
readu,1,rho
close,1

pi=3.14159265

theta=theta-pi/2.

r1=fltarr(nr,nt)
theta1=fltarr(nr,nt)

x=fltarr(nr,nt)
y=fltarr(nr,nt)
for i=0,nr-1 do begin
  for j=0,nt-1 do begin
    x(i,j)=r(i)*sin(theta(j))
    y(i,j)=r(i)*cos(theta(j))
    r1(i,j)=r(i)
    theta1(i,j)=theta(j)
  endfor
endfor

loadct,5

iphi=0

rim1=rmax1
sx=rim1/300.
sy=rim1/300.
r_int1=polar_surface(rho(*,*,iphi),r1,theta1,spacing=[sx,sy],$
bounds=[0,-rim1,rim1,rim1])

rim2=rmax2
sx=rim2/300.
sy=rim2/300.
r_int2=polar_surface(rho(*,*,iphi),r1,theta1,spacing=[sx,sy],$
bounds=[0,-rim2,rim2,rim2])

rim3=rmax3
sx=rim3/300.
sy=rim3/300.
r_int3=polar_surface(rho(*,*,iphi),r1,theta1,spacing=[sx,sy],$
;bounds=[0,-rim3/2,rim3,rim3/2])
bounds=[0,-rim3,rim3,rim3])

minim1=alog10(rhomin1)
maxim1=minim1+range
minim2=alog10(rhomin2)
maxim2=minim2+range
minim3=alog10(rhomin3)
maxim3=minim3+range

;tvscl,r_int1^0.15,0.5,1.,xsize=2.,ysize=4.,/inches
;tvscl,r_int2^0.15,3.,1.,xsize=2.,ysize=4.,/inches
;tvscl,r_int3^0.15,5.5,1.,xsize=2.,ysize=4.,/inches

tv,bytscl(alog10(r_int1),min=minim1,max=maxim1),0.5,1.,xsize=2.,ysize=4.,/inches
tv,bytscl(alog10(r_int2),min=minim2,max=maxim2),3.,1.,xsize=2.,ysize=4.,/inches
tv,bytscl(alog10(r_int3),min=minim3,max=maxim3),5.5,1.,xsize=2.,ysize=4.,/inches


result=size(r_int1)
nx=result(1)
ny=result(2)
x1=findgen(nx)*rim1/float(nx)
y1=findgen(ny)*rim1/float(ny)-rim1

result=size(r_int2)
nx=result(1)
ny=result(2)
x2=findgen(nx)*rim2/float(nx)
y2=findgen(ny)*rim2/float(ny)-rim2

result=size(r_int3)
nx=result(1)
ny=result(2)
x3=findgen(nx)*rim3/float(nx)
y3=findgen(ny)*rim3/float(ny)-rim3


contour,r_int1,x1,y1, POSITION=[0.0625, 0.166666667, 0.3125, 0.833333],$
color=0,/noerase,/nodata,xstyle=1,ystyle=1,xrange=[0,rim1],$
yrange=[-rim1,rim1],$
xminor=2,yminor=2,xticks=3,yticks=6

contour,r_int1,x1,y1, POSITION=[0.0625, 0.166666667, 0.3125, 0.833333],$
color=250,/noerase,/nodata,xstyle=1,ystyle=1,xrange=[0,rim1],$
yrange=[-rim1,rim1],$
xtickname=[' ',' ',' ',' ',' ',' ',' ',' ',' '],$
ytickname=[' ',' ',' ',' ',' ',' ',' ',' ',' '],xminor=2,yminor=2,xticks=3,yticks=6

contour,r_int2,x2,y2, POSITION=[0.375, 0.166666667, 0.625, 0.833333],$
color=0,/noerase,/nodata,xstyle=1,ystyle=1,xrange=[0,rim2],$
yrange=[-rim2,rim2],$
xminor=2,yminor=2

contour,r_int2,x2,y2, POSITION=[0.375, 0.166666667, 0.625, 0.833333],$
color=250,/noerase,/nodata,xstyle=1,ystyle=1,xrange=[0,rim2],$
yrange=[-rim2,rim2],$
xtickname=[' ',' ',' ',' ',' ',' ',' ',' ',' '],$
ytickname=[' ',' ',' ',' ',' ',' ',' ',' ',' '],xminor=2,yminor=2

contour,r_int3,x3,y3, POSITION=[0.6875, 0.166666667, .9375, 0.833333],$
color=0,/noerase,/nodata,xstyle=1,ystyle=1,xrange=[0,rim3],$
yrange=[-rim3,rim3],$
xminor=2,yminor=2,xticks=3,yticks=3

contour,r_int3,x3,y3, POSITION=[0.6875, 0.166666667, .9375, 0.833333],$
color=250,/noerase,/nodata,xstyle=1,ystyle=1,xrange=[0,rim3],$
yrange=[-rim3,rim3],$
xtickname=[' ',' ',' ',' ',' ',' ',' ',' ',' '],$
ytickname=[' ',' ',' ',' ',' ',' ',' ',' ',' '],xminor=2,yminor=2,xticks=3,yticks=3



if !d.name eq 'PS' then device,/close
set_plot,'x'

end




