      SUBROUTINE REAPAR(cflpar,iopar,multid
     &     )

C***********************************************************************
C Subroutine REAPAR handles the reading of input parameters from the
C ".par" file, as well as elementary processing with those input
C parameters to generate arrays.
c
C Original version created by P.J.Flatau, Colorado State Univ.
C Modified by B.T.Draine, Princeton Univ. Obs.
C ported from program DDSCAT by m. wolff
c
C History:
C 95/01/17 (mjw): stripped out of DDSCAT and ported to BLOB
C 99/12/02 (mjw): stripped out of BLOB and ported to MCTHERM
c                 parameter OCCULT defined in calling routine (SETUP)
c 00/03/19 (mjw): added some Vger stuff (Vger position in units of rmax)
c 00/06/30 (baw): redo some spot input
c 00/07/27 (mjw): remove extraneous commas that drove lf95 crazy
c                   (occurrences of "write(*,*),something")
c 2004/01  (baw): lots of I/O changes to make it more user friendly
C
C***********************************************************************
C

      implicit none

C     .. Scalar Arguments ..
      character cflpar*(*)
      integer iopar,multid


C     .. Local Scalars ..
      character cline*70,cmsgnm*70
      character*3 cbub,chole,climb,cpeel,cpoly,cstream
     1   ,cwrite,cspot,cveeg,cnspot,cmultid
      character*5 cscat,cshape
C     ..
C     .. Local Arrays ..
C     ..
C     .. External Subroutines ..
      external errmsg, wrimsg
C     ..
C     .. Intrinsic Functions ..
C     ..
C     .. Data statements ..
C     ..
c     .. Include statements ..

      include 'opacin.txt'
      include 'out.txt'
      include 'random.txt'
      include 'tabl.txt'
      include 'tts.txt'
      include 'spot.txt'
      include 'vger.txt'
      include 'stokes.txt'


C***********************************************************************

      open(unit=iopar,file=cflpar,status='old')

      READ(IOPAR,FMT=*,ERR=40)CLINE
      CALL WRIMSG(' ',CLINE)

C***********************************************************************
C ... Preliminaries
c
      READ(IOPAR,FMT=*,ERR=40)CLINE
      CALL WRIMSG(' ',CLINE)
      READ(IOPAR,FMT=*,ERR=40)np
      WRITE(CMSGNM,FMT='(i11,a)')np
     &     ,' = NP = Number of photons from central star'
      CALL WRIMSG('REAPAR',CMSGNM)

c      READ(IOPAR,FMT=*,ERR=40)npout
c      WRITE(CMSGNM,FMT='(i11,a)')npout
c     &     ,' = NPOUT = Number of photons from outside star'
c      CALL WRIMSG('REAPAR',CMSGNM)
c     no one ever uses this, can put it back in for special cases
      npout=0
      READ(IOPAR,FMT=*,ERR=40)i1
      WRITE(CMSGNM,FMT='(i10,a)')i1
     &     ,' = I1 = random number seed'
      CALL WRIMSG('REAPAR',CMSGNM)
      READ(IOPAR,FMT=*,ERR=40)iwrite
      WRITE(CMSGNM,FMT='(i9,a)')iwrite
     &     ,' = IWRITE = no. of photons between timing calls'
      CALL WRIMSG('REAPAR',CMSGNM)
c      READ(IOPAR,FMT=*,ERR=40)ctherm
c      if ( (ctherm.eq.'YES').OR.(ctherm.eq.'yes') ) then
c         itherm = 1
c         WRITE(CMSGNM,FMT='(a)')
c     &        ,'ITHERM=1, Calculating thermal emission + scattering'
c      else
c         itherm = 0
c         WRITE(CMSGNM,FMT='(a)')
c     &        ,'ITHERM=0, Calculating ONLY scattering'
c      endif
c      CALL WRIMSG('REAPAR',CMSGNM)
      READ(IOPAR,FMT=*,ERR=40)cpeel
      if ( (cpeel.eq.'YES').OR.(cpeel.eq.'yes') ) then
         ipeel = 1
         WRITE(CMSGNM,FMT='(a)')
     &        'IPEEL=1, Using peeling-off procedure'
      else
         ipeel = 0
         WRITE(CMSGNM,FMT='(a)')
     &        'IPEEL=0, Using standard MC (no peeling-off)'
      endif
      CALL WRIMSG('REAPAR',CMSGNM)

c      READ(IOPAR,FMT=*,ERR=40)cveeg
c      if ( (cveeg.eq.'YES').OR.(cveeg.eq.'yes') ) then
c         iveeg = 1
c         WRITE(CMSGNM,FMT='(a)')
c     &        'IVEEG=1, calculating radiation field at point inside'
c      else
c         iveeg = 0
c         WRITE(CMSGNM,FMT='(a)')
c     &        'IVEEG=0, not calculating rad. field at point inside'
c      endif
c      CALL WRIMSG('REAPAR',CMSGNM)
      iveeg=0

      READ(IOPAR,FMT=*,ERR=40)cwrite
      if ( (cwrite.eq.'YES').OR.(cwrite.eq.'yes') ) then
         iwriteim = 1
         WRITE(CMSGNM,FMT='(a)')
     &        'IWRITEIM=1, writing out all images'
      else
         iwriteim = 0
         WRITE(CMSGNM,FMT='(a)')
     &        'IWRITEIM=0, only writing peeling-off images'
      endif
      CALL WRIMSG('REAPAR',CMSGNM)
      READ(IOPAR,FMT=*,ERR=40)climb
      if ( (climb.eq.'YES').OR.(climb.eq.'yes') ) then
         limb = 1
         WRITE(CMSGNM,FMT='(a)')
     &        'LIMB=1, Limb-darkening for central source'
      else
         limb = 0
         WRITE(CMSGNM,FMT='(a)')
     &        'LIMB=0, NO limb-darkening for central source'
      endif
      CALL WRIMSG('REAPAR',CMSGNM)

C***********************************************************************
C ... Dust preliminaries
      READ(IOPAR,FMT=*,ERR=40)CLINE
      CALL WRIMSG(' ',CLINE)

      READ(IOPAR,FMT=*,ERR=40)cmultid
      WRITE(CMSGNM,FMT='(A,A)')
     $    cmultid,' - different dust in diff. regions?'
      CALL WRIMSG('REAPAR',CMSGNM)
      if (cmultid.eq.'YES'.or.cmultid.eq.'yes') then
         multid=1
      else
         multid=0
      endif

      READ(IOPAR,FMT=*,ERR=40)cscat
      WRITE(CMSGNM,FMT='(A,A)')
     $    cscat,' - type of scattering'
      CALL WRIMSG('REAPAR',CMSGNM)
      IF(cscat.ne.'ANALY'.and.cscat.ne.'TABLE')then
         CALL ERRMSG('FATAL','REAPAR',
     $        'Check value of CSCAT*5 in .PAR file')
      else
         if(cscat.eq.'ANALY') idust=1
         if (cscat.eq.'TABLE') idust=2
      endif


C***********************************************************************
C ... Central source properties
c
      READ(IOPAR,FMT=*,ERR=40)CLINE
      CALL WRIMSG(' ',CLINE)
      READ(IOPAR,FMT=*,ERR=40)rstar
      WRITE(CMSGNM,FMT='(1pe13.6,a)')rstar
     &     ,' = RSTAR = Stellar radius in Solar radii'
      CALL WRIMSG('REAPAR',CMSGNM)
      READ(IOPAR,FMT=*,ERR=40)tstar
      WRITE(CMSGNM,FMT='(1pe13.6,a)')tstar
     &     ,' = TSTAR = Blackbody temperature of central star (K)'
      CALL WRIMSG('REAPAR',CMSGNM)

      READ(IOPAR,FMT=*,ERR=40)massc
      WRITE(CMSGNM,FMT='(1pe13.6,a)')massc
     &     ,' = MASSC = Mass of central star (for TSC properties)'
      CALL WRIMSG('REAPAR',CMSGNM)

      READ(IOPAR,FMT=*,ERR=40) cspot
      if ( (cspot.eq.'YES').OR.(cspot.eq.'yes') ) then
         ispot = 1
         WRITE(CMSGNM,FMT='(a)')
     &        'ISPOT=1, emission from star+spot'
	   spotflag = 1   !require to run spotset first time
      else
         ispot = 0
         WRITE(CMSGNM,FMT='(a)')
     &        'ISPOT=0, uniform emission from star'
      endif
      CALL WRIMSG('REAPAR',CMSGNM)

      READ(IOPAR,FMT=*,ERR=40)tspot
      WRITE(CMSGNM,FMT='(1pe13.6,a)')tspot
     &     ,' = TSTAR = Blackbody temperature of spot (K)'
      CALL WRIMSG('REAPAR',CMSGNM)

      READ(IOPAR,FMT=*,ERR=40)thspot
      WRITE(CMSGNM,FMT='(1pe13.6,a)')thspot
     &     ,' = THSPOT = radius of spot in degrees'
      CALL WRIMSG('REAPAR',CMSGNM)

      READ(IOPAR,FMT=*,ERR=40)thspotin
      WRITE(CMSGNM,FMT='(1pe13.6,a)')thspotin
     &     ,' = THSPOTIN = inner radius of spot (for ring)'
      CALL WRIMSG('REAPAR',CMSGNM)

      READ(IOPAR,FMT=*,ERR=40)spotlat
      WRITE(CMSGNM,FMT='(1pe13.6,a)')spotlat
     &     ,' = SPOTLAT = latitude of spot (degrees)'
      CALL WRIMSG('REAPAR',CMSGNM)

      READ(IOPAR,FMT=*,ERR=40)spotlon
      WRITE(CMSGNM,FMT='(1pe13.6,a)')spotlon
     &     ,' = SPOTLON = longitude of spot (degrees)'
      CALL WRIMSG('REAPAR',CMSGNM)

      READ(IOPAR,FMT=*,ERR=40) cnspot
      if ( (cnspot.eq.'ONE').OR.(cnspot.eq.'one') ) then
         nspot = 1
         WRITE(CMSGNM,FMT='(a)')
     &        'NSPOT=1, one spot'
      else
         nspot = 2
         WRITE(CMSGNM,FMT='(a)')
     &        'NSPOT=2, two spots'
      endif
      CALL WRIMSG('REAPAR',CMSGNM)

C***********************************************************************
C ... Disk properties
c
      READ(IOPAR,FMT=*,ERR=40)CLINE
      CALL WRIMSG(' ',CLINE)
      READ(IOPAR,FMT=*,ERR=40)massd
      WRITE(CMSGNM,FMT='(1pe13.6,a)')massd
     &     ,' = MASSD = Disk mass in solar masses'
      CALL WRIMSG('REAPAR',CMSGNM)
      READ(IOPAR,FMT=*,ERR=40)rmaxd
      WRITE(CMSGNM,FMT='(1pe13.6,a)')rmaxd
     &     ,' = RMAXD = Maximum disk radius'
      CALL WRIMSG('REAPAR',CMSGNM)
      READ(IOPAR,FMT=*,ERR=40)rmind
      WRITE(CMSGNM,FMT='(1pe13.6,a)')rmind
     &     ,' = RMIND = Maximum disk radius'
      CALL WRIMSG('REAPAR',CMSGNM)
c      READ(IOPAR,FMT=*,ERR=40)rddust
c      WRITE(CMSGNM,FMT='(1pe13.6,a)')rddust
c     &     ,' = RDDUST = Minimum disk opacity radius in Rstar'
c      CALL WRIMSG('REAPAR',CMSGNM)
      rddust=rmind
      READ(IOPAR,FMT=*,ERR=40)zmin
      WRITE(CMSGNM,FMT='(1pe13.6,a)')zmin
     &     ,' = ZMIN = Scale height of disk at Rstar'
      CALL WRIMSG('REAPAR',CMSGNM)
      READ(IOPAR,FMT=*,ERR=40)a
      WRITE(CMSGNM,FMT='(1pe13.6,a)')a
     &     ,' = A = Disk density exponent (~ r^(-a))'
      CALL WRIMSG('REAPAR',CMSGNM)
      READ(IOPAR,FMT=*,ERR=40)b
      WRITE(CMSGNM,FMT='(1pe13.6,a)')b
     &     ,' = B = Disk scale height exponent (~ r^(b))'
      CALL WRIMSG('REAPAR',CMSGNM)

C***********************************************************************
C ... Envelope properties
c
      READ(IOPAR,FMT=*,ERR=40)CLINE
      CALL WRIMSG(' ',CLINE)

      READ(IOPAR,FMT=*,ERR=40)rmax
      WRITE(CMSGNM,FMT='(1pe13.6,a)')rmax
     &     ,' = RMAX = Maximum envelope radius in AU'
      CALL WRIMSG('REAPAR',CMSGNM)
      READ(IOPAR,FMT=*,ERR=40)rmine
      WRITE(CMSGNM,FMT='(1pe13.6,a)')rmine
     &     ,' = RMINE = Minimum envelope radius in AU'
      CALL WRIMSG('REAPAR',CMSGNM)
      READ(IOPAR,FMT=*,ERR=40)rate
      WRITE(CMSGNM,FMT='(1pe13.6,a)')rate
     &     ,' = RATE = Mass infall rate for TSC env.'//
     &     '(M_O/yr)'
      CALL WRIMSG('REAPAR',CMSGNM)
      READ(IOPAR,FMT=*,ERR=40)rc
      WRITE(CMSGNM,FMT='(1pe13.6,a)')rc
     &     ,' = RC = centrifugal radius for TSC env.'
      CALL WRIMSG('REAPAR',CMSGNM)
      READ(IOPAR,FMT=*,ERR=40)chole
      READ(IOPAR,FMT=*,ERR=40)cshape
      print*,'cshape ',cshape
      READ(IOPAR,FMT=*,ERR=40)rchole      
      READ(IOPAR,FMT=*,ERR=40)thet1
      READ(IOPAR,FMT=*,ERR=40)thet2
      READ(IOPAR,FMT=*,ERR=40)ex1
      READ(IOPAR,FMT=*,ERR=40)ex2
      READ(IOPAR,FMT=*,ERR=40)z01
      READ(IOPAR,FMT=*,ERR=40)z02
      READ(IOPAR,FMT=*,ERR=40)exf
      READ(IOPAR,FMT=*,ERR=40)rhoconst1
      READ(IOPAR,FMT=*,ERR=40)rhoconst2
      READ(IOPAR,FMT=*,ERR=40)rhoamb
      READ(IOPAR,FMT=*,ERR=40)zflowmin

      if ( (chole.eq.'YES').OR.(chole.eq.'yes') ) then
         ihole = 1
         WRITE(CMSGNM,FMT='(a)')
     &        'IHOLE=1, Carving cavity out'
         CALL WRIMSG('REAPAR',CMSGNM)
         WRITE(CMSGNM,FMT='(f6.1,a)')thet1
     &        ,' = THET1 = Opening angle of inner cavity wall'
         CALL WRIMSG('REAPAR',CMSGNM)
         WRITE(CMSGNM,FMT='(f6.1,a)')thet2
     &        ,' = THET2 = Opening angle of outer cavity wall'
         CALL WRIMSG('REAPAR',CMSGNM)
         
         WRITE(CMSGNM,FMT='(f7.2,a)')ex1
     &        ,' = EX1 = Inner Cavity wall exponent'
         CALL WRIMSG('REAPAR',CMSGNM)
         
         if (cshape.eq.'STREA') then
            thetmu0=thet2
            ipoly=0
            istream=1
            WRITE(CMSGNM,FMT='(a)')
     &           'outer wall is streamline shape cavity'
            WRITE(CMSGNM,FMT='(1pe11.4,a)')rchole
     &           ,' = RCHOLE = Streamline hole size in AU'
            CALL WRIMSG('REAPAR',CMSGNM)
            WRITE(CMSGNM,FMT='(f6.2,a)')thetmu0
     &           ,' = THETMU0 = Opening angle of streamline hole (deg)'
         else
            ipoly = 1
            istream=0
            WRITE(CMSGNM,FMT='(a)')
     &           'Polynomial-shaped cavity, inner and outer'
            WRITE(CMSGNM,FMT='(f7.2,a)')ex2
     &           ,' = EX2 = Outer Cavity wall exponent'
            CALL WRIMSG('REAPAR',CMSGNM)
         endif
         
         WRITE(CMSGNM,FMT='(1pe11.4,a)')z01
     &        ,' = Z01 = Height of inner wall at w=0'
         CALL WRIMSG('REAPAR',CMSGNM)
         WRITE(CMSGNM,FMT='(1pe11.4,a)')z02
     &        ,' = Z02 = Height of outer wall at w=0'
         CALL WRIMSG('REAPAR',CMSGNM)
         WRITE(CMSGNM,FMT='(f7.2,a)')exf
     &        ,' = EXF = exponent for cavity density power-law'
         CALL WRIMSG('REAPAR',CMSGNM)
         WRITE(CMSGNM,FMT='(1pe11.4,a)')rhoconst1
     &        ,' = RHOCONST1 = Coefficient for inner cavity density'
         CALL WRIMSG('REAPAR',CMSGNM)
         WRITE(CMSGNM,FMT='(1pe11.4,a)')rhoconst2
     &        ,' = RHOCONST2 = Coefficient for outer cavity density'
         CALL WRIMSG('REAPAR',CMSGNM)
         WRITE(CMSGNM,FMT='(1pe11.4,a)')rhoamb
     &        ,' = RHOAMB = ambient density'
         CALL WRIMSG('REAPAR',CMSGNM)
c         if ( (cstream.eq.'YES').OR.(cstream.eq.'yes') ) then
c            istream = 1
c            WRITE(CMSGNM,FMT='(a)')
c     &           'ISTREAM=1, streamline hole'
c            CALL WRIMSG('REAPAR',CMSGNM)
c            
c         else
c            istream = 0
c            WRITE(CMSGNM,FMT='(a)')
c     &           'ISTREAM=0'
c         endif
      else
         ihole = 0
         WRITE(CMSGNM,FMT='(a)')
     &        'IHOLE=0, NO cavity'
      endif
      CALL WRIMSG('REAPAR',CMSGNM)
      
      WRITE(CMSGNM,FMT='(1pe11.4,a)')zflowmin
     &     ,' = minimum radius of outflow (AU)'
      CALL WRIMSG('REAPAR',CMSGNM)
      
      
c     no bubble for now
c      READ(IOPAR,FMT=*,ERR=40)cbub
c      READ(IOPAR,FMT=*,ERR=40)nbub
c      READ(IOPAR,FMT=*,ERR=40)zbub1
c      READ(IOPAR,FMT=*,ERR=40)zbub2
c      READ(IOPAR,FMT=*,ERR=40)buboa
c      if ( (cbub.eq.'YES').OR.(cbub.eq.'yes') ) then
c         ibub = 1
c         WRITE(CMSGNM,FMT='(a)')
c     &        'IBUB=1, including bubble'
c         CALL WRIMSG('REAPAR',CMSGNM)
c         WRITE(CMSGNM,FMT='(f7.2,a)')nbub
c     &        ,' = NBUB = Exponent describing bubble shape'
c         CALL WRIMSG('REAPAR',CMSGNM)
c         WRITE(CMSGNM,FMT='(1pe11.4,a)')zbub1
c     &        ,' = ZBUB1 = height of bubble inner wall'
c         CALL WRIMSG('REAPAR',CMSGNM)
c         WRITE(CMSGNM,FMT='(1pe11.4,a)')zbub2
c     &        ,' = ZBUB2 = height of bubble outer wall'
c         CALL WRIMSG('REAPAR',CMSGNM)
c         WRITE(CMSGNM,FMT='(1pe11.4,a)')buboa
c     &        ,' = BUBOA = Opening angle of hole in bubble'
c      else
c         ibub = 0
c         WRITE(CMSGNM,FMT='(a)')
c     &        'IBUB=0, NO bubble'
c      endif
c      CALL WRIMSG('REAPAR',CMSGNM)
      ibub=0
      nbub=2
      zbub1=230.
      zbub2=260.
      buboa=2.

C***********************************************************************
C ... Vger stuff
c
c XVG0,YVG0,ZVG0 = location of VGER in units of RMAX
c XSRC0,YSRC0,ZSRC0 = location of outer illum. (RMAX)
c ISRC0,QSRC0,USRC0,VSRC0 = normalized stokes for incident outside illum.
c
c     don't include for now.

c      READ(IOPAR,FMT=*,ERR=40)CLINE
c      CALL WRIMSG(' ',CLINE)
c      READ(IOPAR,FMT=*,ERR=40)xvg0,yvg0,zvg0
c      READ(IOPAR,FMT=*,ERR=40)xsrc0,ysrc0,zsrc0
c      READ(IOPAR,FMT=*,ERR=40)isrc0,qsrc0,usrc0,vsrc0
c      if (CVEEG.eq.'YES') then
c         write(CMSGNM,fmt='(3(1pe11.4,1x),a)') xvg0,yvg0,zvg0,
c     &        ' - VGER location in units of RMAX'
c         CALL WRIMSG('REAPAR',CMSGNM)
c         write(CMSGNM,fmt='(3(1pe11.4,1x),a)') xsrc0,ysrc0,zsrc0,
c     &        ' - src illum. in units of RMAX'
c         CALL WRIMSG('REAPAR',CMSGNM)
c         write(CMSGNM,fmt='(4(1pe11.4,1x),a)')isrc0,qsrc0,usrc0,
c     &        vsrc0,
c     &       ' - incident stokes'
c         CALL WRIMSG('REAPAR',CMSGNM)
c      endif
      xvg0=0.4
      yvg0=0.4
      zvg0=-0.2
      xsrc0=1.4
      ysrc0=0.0
      zsrc0=1.2
      isrc0=1.
      qsrc0=0.
      usrc0=0.
      
      
C***********************************************************************
C ... output data
c
      READ(IOPAR,FMT=*,ERR=40)CLINE
      CALL WRIMSG(' ',CLINE)
      READ(IOPAR,FMT=*,ERR=40)rmaxi
      WRITE(CMSGNM,FMT='(1pe13.6,a)')rmaxi
     &     ,' = RMAXI = Image half-size in AU'
      CALL WRIMSG('REAPAR',CMSGNM)
      READ(IOPAR,FMT=*,ERR=40)aperture2
      WRITE(CMSGNM,FMT='(3(f8.1,1x),a)')aperture2
     &     ,' = APERTURE(1-3) = radius of aper 1-3 in AU'
      CALL WRIMSG('REAPAR',CMSGNM)
      READ(IOPAR,FMT=*,ERR=40)thete
      WRITE(CMSGNM,FMT='(f7.2,a)')thete
     &     ,' = THETE = theta angle (deg) of high S/N image'
      CALL WRIMSG('REAPAR',CMSGNM)

c     leave this out for 2-D code
      READ(IOPAR,FMT=*,ERR=40)phie
      WRITE(CMSGNM,FMT='(f7.2,a)')phie
     &     ,' = PHIE = phi angle (deg) of high S/N image'
      CALL WRIMSG('REAPAR',CMSGNM)

c      READ(IOPAR,FMT=*,ERR=40)xymaxdens
c      WRITE(CMSGNM,FMT='(f7.2,a)')xymaxdens
c     &     ,' =XYMAXDENS = xmax for xydens.dat in units of rstar'
c      CALL WRIMSG('REAPAR',CMSGNM)
      xymaxdens=500000.

      CLOSE (IOPAR)
      RETURN
   40 CONTINUE
      CALL ERRMSG('FATAL','REAPAR',' Error reading .par file')
      RETURN
      END




